@@ -6,6 +6,7 @@ Examining feeds and extracting links for further processing.
 ## under GNU GPL v3 license
 
 import logging
+import json
 import re
 
 from courlan import check_url, clean_url, fix_relative_urls, get_hostinfo, validate_url
@@ -15,11 +16,11 @@ from .utils import filter_urls, load_html
 
 LOGGER = logging.getLogger(__name__)
 
-FEED_TYPES = set(['application/atom+xml', 'application/rdf+xml', 'application/rss+xml', 'application/x.atom+xml', 'application/x-atom+xml', 'text/atom+xml', 'text/rdf+xml', 'text/rss+xml', 'text/xml'])
+FEED_TYPES = set(['application/atom+xml', 'application/json', 'application/rdf+xml', 'application/rss+xml', 'application/x.atom+xml', 'application/x-atom+xml', 'text/atom+xml', 'text/plain', 'text/rdf+xml', 'text/rss+xml', 'text/xml'])
 FEED_OPENING = re.compile(r'<(feed|rss|\?xml)')
 LINK_ATTRS = re.compile(r'<link .*?href=".+?"')
 LINK_HREF = re.compile(r'href="(.+?)"')
-LINK_ELEMENTS = re.compile(r'<link>(?:<!\[CDATA\[)?(.+?)(?:\]\]>)?</link>')
+LINK_ELEMENTS = re.compile(r'<link>(?:\s*)(?:<!\[CDATA\[)?(.+?)(?:\]\]>)?(?:\s*)</link>')
 BLACKLIST = re.compile(r'\bcomments\b')  # no comment feed
 
 
@@ -28,7 +29,7 @@ def handle_link_list(linklist, domainname, baseurl, target_lang=None):
        lead to a web page'''
     output_links = []
     # sort and uniq
-    for item in sorted(list(set(linklist))):
+    for item in sorted(set(linklist)):
         # fix and check
         link = fix_relative_urls(baseurl, item)
         # control output for validity
@@ -48,9 +49,27 @@ def extract_links(feed_string, domainname, baseurl, reference, target_lang=None)
     feed_links = []
     # check if it's a feed
     if feed_string is None:
+        LOGGER.debug('Empty feed: %s', domainname)
         return feed_links
     feed_string = feed_string.strip()
-    if not FEED_OPENING.match(feed_string):
+    # typical first and second lines absent
+    if not FEED_OPENING.match(feed_string) and not \
+        ('<rss' in feed_string[:100] or '<feed' in feed_string[:100]):
+        # could be JSON
+        if feed_string.startswith('{'):
+            try:
+                feed_dict = json.loads(feed_string)
+                if 'items' in feed_dict:
+                    for item in feed_dict['items']:
+                        if 'url' in item:
+                            feed_links.append(item['url'])
+                        # fallback: https://www.jsonfeed.org/version/1.1/
+                        elif 'id' in item:
+                            feed_links.append(item['id'])
+            except json.decoder.JSONDecodeError:
+                LOGGER.debug('JSON decoding error: %s', domainname)
+        else:
+            LOGGER.debug('Possibly invalid feed: %s', domainname)
         return feed_links
     # could be Atom
     if '<link ' in feed_string:
@@ -58,13 +77,16 @@ def extract_links(feed_string, domainname, baseurl, reference, target_lang=None)
             if 'atom+xml' in link or 'rel="self"' in link:
                 continue
             feedlink = LINK_HREF.search(link).group(1)
-            if '"' in feedlink:
-                feedlink = feedlink.split('"')[0]
+            #if '"' in feedlink:
+            #    feedlink = feedlink.split('"')[0]
             feed_links.append(feedlink)
     # could be RSS
     elif '<link>' in feed_string:
-        for item in LINK_ELEMENTS.findall(feed_string, re.DOTALL):
-            feed_links.append(item.strip())
+        feed_links.extend(
+            item.strip()
+            for item in LINK_ELEMENTS.findall(feed_string, re.DOTALL)
+        )
+
     # refine
     output_links = handle_link_list(feed_links, domainname, baseurl, target_lang)
     output_links = [l for l in output_links if l != reference and l.count('/') > 2]
@@ -83,11 +105,12 @@ def determine_feed(htmlstring, baseurl, reference):
     tree = load_html(htmlstring)
     # safeguard
     if tree is None:
+        LOGGER.debug('Invalid HTML/Feed page: %s', baseurl)
         return []
     feed_urls = []
     for linkelem in tree.xpath('//link[@rel="alternate"]'):
         # discard elements without links
-        if not 'href' in linkelem.attrib:
+        if 'href' not in linkelem.attrib:
             continue
         # most common case
         if 'type' in linkelem.attrib and linkelem.get('type') in FEED_TYPES:
@@ -96,7 +119,7 @@ def determine_feed(htmlstring, baseurl, reference):
         elif 'atom' in linkelem.get('href') or 'rss' in linkelem.get('href'):
             feed_urls.append(linkelem.get('href'))
     # backup
-    if len(feed_urls) == 0:
+    if not feed_urls:
         for linkelem in tree.xpath('//a[@href]'):
             if linkelem.get('href')[-4:].lower() in ('.rss', '.rdf', '.xml'):
                 feed_urls.append(linkelem.get('href'))
@@ -154,16 +177,24 @@ def find_feed_urls(url, target_lang=None):
             feed_links = filter_urls(feed_links, urlfilter)
             LOGGER.debug('%s feed links found for %s', len(feed_links), domainname)
             return feed_links
+        LOGGER.debug('No usable feed links found: %s', url)
     else:
         LOGGER.warning('Could not download web page: %s', url)
+        if url.strip('/') != baseurl:
+            return try_homepage(baseurl, target_lang)
     # try alternative: Google News
     if target_lang is not None:
-        url = 'https://news.google.com/rss/search?q=site:' + baseurl + '&hl=' + target_lang + '&scoring=n&num=100'
-        downloaded = fetch_url(url)
+        downloaded = fetch_url('https://news.google.com/rss/search?q=site:' + baseurl + '&hl=' + target_lang + '&scoring=n&num=100')
         if downloaded is not None:
             feed_links = extract_links(downloaded, domainname, baseurl, url, target_lang)
             feed_links = filter_urls(feed_links, urlfilter)
-            LOGGER.debug('%s feed links found for %s', len(feed_links), domainname)
+            LOGGER.debug('%s Google news links found for %s', len(feed_links), domainname)
             return feed_links
-        LOGGER.warning('Could not download web page: %s', url)
     return []
+
+
+def try_homepage(baseurl, target_lang):
+    '''Shift into reverse and try the homepage instead of the particular feed
+       page that was given as input.'''
+    LOGGER.info('Probing homepage for feeds instead: %s', baseurl)
+    return find_feed_urls(baseurl, target_lang)
