@@ -3,14 +3,15 @@ Functions related to content filtering, mostly duplicate detection and language
 detection.
 """
 
-import base64
-import hashlib
 import logging
 import re
 
+from base64 import b64encode
+from hashlib import sha1
+
 # language detection
 try:
-    import cld3
+    import py3langid
     LANGID_FLAG = True
 except ImportError:
     LANGID_FLAG = False
@@ -26,7 +27,11 @@ LRU_TEST = LRUCache(maxsize=LRU_SIZE)
 
 RE_HTML_LANG = re.compile(r'([a-z]{2})', re.I)
 
-RE_FILTER = re.compile(r'\W*(Drucken|E-?Mail|Facebook|Flipboard|Google|Instagram|Linkedin|Mail|PDF|Pinterest|Pocket|Print|Reddit|Twitter|Whatsapp|Xing|Mehr zum Thema:?|More on this.{,8}$)$', flags=re.IGNORECASE)
+# Mostly filters for social media
+RE_FILTER = re.compile(r'\W*(Drucken|E-?Mail|Facebook|Flipboard|Google|Instagram|'
+                        'Linkedin|Mail|PDF|Pinterest|Pocket|Print|QQ|Reddit|Twitter|'
+                        'WeChat|WeiBo|Whatsapp|Xing|Mehr zum Thema:?|More on this.{,8}$)$',
+                       flags=re.IGNORECASE)
 # COMMENTS_BLACKLIST = ('( Abmelden / Ändern )') # Fill in your details below|Trage deine Daten unten|Kommentar verfassen|Bitte logge dich|Hinterlasse einen Kommentar| to %s| mit %s)
 
 
@@ -56,41 +61,49 @@ def duplicate_test(element, config):
     return False
 
 
-def check_html_lang(tree, target_language):
+def check_html_lang(tree, target_language, strict=False):
     '''Check HTML meta-elements for language information and split
        the result in case there are several languages'''
     # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Language
-    target_elements = tree.xpath('//html[@lang]')
-    if len(target_elements) > 0:
-        for elem in target_elements:
-            if target_language in RE_HTML_LANG.split(elem.get('lang')):
-                return True
-        LOGGER.warning('HTML lang detection failed')
-        return False
     target_elements = tree.xpath('//meta[@http-equiv="content-language"]')
     if len(target_elements) > 0:
         for elem in target_elements:
-            if target_language in RE_HTML_LANG.split(elem.get('content')):
+            if target_language in RE_HTML_LANG.split(elem.get('content').lower()):
                 return True
-        LOGGER.warning('HTML lang detection failed')
+        LOGGER.debug('HTML lang detection failed')
         return False
-    LOGGER.info('No HTML lang elements found')
+    # locale
+    target_elements = tree.xpath('//meta[@property="og:locale"]')
+    if len(target_elements) > 0:
+        for elem in target_elements:
+            if target_language in RE_HTML_LANG.split(elem.get('content').lower()):
+                return True
+        LOGGER.debug('HTML lang detection failed')
+        return False
+    # HTML lang attribute: sometimes a wrong indication
+    if strict is True:
+        target_elements = tree.xpath('//html[@lang]')
+        if len(target_elements) > 0:
+            for elem in target_elements:
+                if target_language in RE_HTML_LANG.split(elem.get('lang')):
+                    return True
+            LOGGER.debug('HTML lang detection failed')
+            return False
+    LOGGER.info('No relevant lang elements found')
     return True
 
 
 def language_filter(temp_text, temp_comments, target_language, docmeta):
     '''Run external component (if installed) for language identification'''
-    # sanity check on language
     if target_language is not None:
         if LANGID_FLAG is True:
-            # comments
-            if len(temp_comments) > len(temp_text):
-                result = cld3.get_language(temp_comments)
-            # default
-            else:
-                result = cld3.get_language(temp_text)
-            if result.language != target_language:
-                LOGGER.warning('wrong language: %s %s %s', result, docmeta['id'], docmeta['url'])
+            result, _ = (
+                py3langid.classify(temp_text)
+                if len(temp_text) > len(temp_comments)
+                else py3langid.classify(temp_comments)
+            )
+            if result != target_language:
+                LOGGER.warning('wrong language: %s %s %s', result, docmeta.id, docmeta.url)
                 return True
         else:
             LOGGER.warning('Detector not installed, no language detection run')
@@ -106,26 +119,21 @@ def textfilter(element):
         testtext = element.text
     if text_chars_test(testtext) is False:
         return True
-    for line in testtext.splitlines():
-        #if len(line) <= 5:
-        #    continue
-        if RE_FILTER.match(line):
-            return True
-    return False
+    # to check: line len → continue if len(line) <= 5
+    return any(RE_FILTER.match(line) for line in testtext.splitlines())
 
 
 def text_chars_test(string):
     '''Determine if a string is only composed of spaces and/or control characters'''
     # or not re.search(r'\w', string)
-    if string is None or len(string) == 0 or string.isspace():
-        return False
-    return True
+    # return string is not None and len(string) != 0 and not string.isspace()
+    return string not in (None, '') and not string.isspace()
 
 
 def content_fingerprint(string):
     '''Calculate a hash value for meaningful bits of the content'''
     teststring = ' '.join(re.findall(r'\w{5,}', string.lower()))
-    m = hashlib.sha1()
+    m = sha1()
     m.update(teststring.encode())
     fingerprint = m.digest()
-    return base64.b64encode(fingerprint).decode()
+    return b64encode(fingerprint).decode()
