@@ -61,6 +61,7 @@ def sitemap_search(url, target_lang=None):
         if len(url) > len(baseurl) + 2:
             urlfilter = url
     sitemapurls, linklist = download_and_process_sitemap(sitemapurl, domainname, baseurl, target_lang)
+    sitemaps_seen = {sitemapurl}
     if sitemapurls == [] and len(linklist) > 0:
         linklist = filter_urls(linklist, urlfilter)
         LOGGER.debug('%s sitemap links found for %s', len(linklist), domainname)
@@ -69,12 +70,16 @@ def sitemap_search(url, target_lang=None):
     if sitemapurls == [] and linklist == []:
         sitemapurls = find_robots_sitemaps(baseurl)
         # try additional URLs just in case
-        if sitemapurls == []:
+        if not sitemapurls:
             sitemapurls = [''.join([baseurl, '/', g]) for g in GUESSES]
     # iterate through nested sitemaps and results
     i = 1
     while sitemapurls:
-        sitemapurls, linklist = download_and_process_sitemap(sitemapurls.pop(), domainname, baseurl, target_lang, sitemapurls, linklist)
+        sitemapurl = sitemapurls.pop()
+        sitemapurls, linklist = download_and_process_sitemap(sitemapurl, domainname, baseurl, target_lang, sitemapurls, linklist)
+        # sanity check: keep track of visited sitemaps and exclude them
+        sitemaps_seen.add(sitemapurl)
+        sitemapurls = [s for s in sitemapurls if s not in sitemaps_seen]
         # counter and safeguard
         i += 1
         if i > MAX_SITEMAPS_SEEN:
@@ -91,7 +96,8 @@ def check_sitemap(url, contents):
         # strip query and fragments
         url = SCRUB_REGEX.sub('', url)
         if POTENTIAL_SITEMAP.search(url) and \
-            (not isinstance(contents, str) or not SITEMAP_FORMAT.match(contents)):
+            (not isinstance(contents, str) or not SITEMAP_FORMAT.match(contents)) \
+             or '<html' in contents[:150].lower():
             LOGGER.info('not a valid XML sitemap: %s', url)
             return None
     return contents
@@ -141,20 +147,15 @@ def handle_link(link, sitemapurl, domainname, baseurl, target_lang):
     link = fix_relative_urls(baseurl, link)
     # clean and normalize
     link = clean_url(link, target_lang)
-    if link is not None:
-        if lang_filter(link, target_lang) is True:
-            newdomain = extract_domain(link)
-            if newdomain is not None:
-                # don't take links from another domain and make an exception for main platforms
-                if newdomain != domainname and not WHITELISTED_PLATFORMS.search(newdomain):
-                    LOGGER.warning('Diverging domain names: %s %s', domainname, newdomain)
-                else:
-                    if DETECT_SITEMAP_LINK.search(link):
-                        state = 'sitemap'
-                    else:
-                        state = 'link'
-            else:
-                LOGGER.error("Couldn't extract domain: %s", link)
+    if link is not None and lang_filter(link, target_lang) is True:
+        newdomain = extract_domain(link)
+        if newdomain is None:
+            LOGGER.error("Couldn't extract domain: %s", link)
+        # don't take links from another domain and make an exception for main platforms
+        elif newdomain != domainname and not WHITELISTED_PLATFORMS.search(newdomain):
+            LOGGER.warning('Diverging domain names: %s %s', domainname, newdomain)
+        else:
+            state = 'sitemap' if DETECT_SITEMAP_LINK.search(link) else 'link'
     return link, state
 
 
@@ -170,7 +171,7 @@ def store_sitemap_link(sitemapurls, linklist, link, state):
 
 def extract_sitemap_langlinks(pagecontent, sitemapurl, domainname, baseurl, target_lang):
     'Extract links corresponding to a given target language.'
-    if not 'hreflang=' in pagecontent:
+    if 'hreflang=' not in pagecontent:
         return [], []
     sitemapurls, linklist = [], []
     # compile regex here for modularity and efficiency
